@@ -76,7 +76,7 @@ static void init_mounted_dev_list (int mandatory);
 #endif
 
 static void process_top_path PARAMS((char *pathname, mode_t mode));
-static int process_path PARAMS((char *pathname, char *name, boolean leaf, char *parent, mode_t type));
+static int process_path PARAMS((char *pathname, char *name, bool leaf, char *parent, mode_t type));
 static void process_dir PARAMS((char *pathname, char *name, int pathlen, const struct stat *statp, char *parent));
 
 
@@ -110,12 +110,13 @@ enum WdSanityCheckFatality
   };
 
 
-int get_current_dirfd (void)
+int
+get_current_dirfd (void)
 {
   return AT_FDCWD;
 }
 
-
+
 int
 main (int argc, char **argv)
 {
@@ -135,6 +136,7 @@ main (int argc, char **argv)
       remember_non_cloexec_fds ();
     }
 
+  state.already_issued_stat_error_msg = false;
   state.shared_files = sharefile_init ("w");
   if (NULL == state.shared_files)
     {
@@ -243,15 +245,15 @@ main (int argc, char **argv)
   cleanup ();
   return state.exit_status;
 }
-
-boolean is_fts_enabled (int *ftsoptions)
+
+bool is_fts_enabled (int *ftsoptions)
 {
   /* this version of find (i.e. this main ()) does not use fts. */
   *ftsoptions = 0;
   return false;
 }
 
-
+
 static char *
 specific_dirname (const char *dir)
 {
@@ -280,7 +282,7 @@ specific_dirname (const char *dir)
 }
 
 
-
+
 /* Return non-zero if FS is the name of a file system that is likely to
  * be automounted
  */
@@ -450,7 +452,7 @@ dirchange_is_fatal (const char *specific_what,
  * Hence we print a warning message to indicate that the output of find
  * might be inconsistent due to the change in the file system.
  */
-static boolean
+static bool
 wd_sanity_check (const char *thing_to_stat,
 		const char *progname,
 		const char *what,
@@ -461,7 +463,7 @@ wd_sanity_check (const char *thing_to_stat,
 		int line_no,
 		enum TraversalDirection direction,
 		enum WdSanityCheckFatality isfatal,
-		boolean *changed) /* output parameter */
+		bool *changed) /* output parameter */
 {
   const char *fstype;
   char *specific_what = NULL;
@@ -472,7 +474,7 @@ wd_sanity_check (const char *thing_to_stat,
 
   set_stat_placeholders (newinfo);
   if ((*options.xstat) (current_dir, newinfo) != 0)
-    fatal_file_error (thing_to_stat);
+    fatal_target_file_error (errno, thing_to_stat);
 
   if (old_dev != newinfo->st_dev)
     {
@@ -553,7 +555,7 @@ wd_sanity_check (const char *thing_to_stat,
 
   return true;
 }
-
+
 enum SafeChdirStatus
   {
     SafeChdirOK,
@@ -578,13 +580,13 @@ safely_chdir_lstat (const char *dest,
 		    enum TraversalDirection direction,
 		    struct stat *statbuf_dest,
 		    enum ChdirSymlinkHandling symlink_follow_option,
-		    boolean *did_stat)
+		    bool *did_stat)
 {
   struct stat statbuf_arrived;
   int rv, dotfd=-1;
   int saved_errno;		/* specific_dirname() changes errno. */
-  boolean rv_set = false;
-  boolean statflag = false;
+  bool rv_set = false;
+  bool statflag = false;
   int tries = 0;
   enum WdSanityCheckFatality isfatal = RETRY_IF_SANITY_CHECK_FAILS;
 
@@ -675,7 +677,7 @@ safely_chdir_lstat (const char *dest,
 	  if (0 == chdir (dest))
 	    {
 	      /* check we ended up where we wanted to go */
-	      boolean changed = false;
+	      bool changed = false;
 	      if (!wd_sanity_check (".", program_name, ".",
 				    statbuf_dest->st_dev,
 				    statbuf_dest->st_ino,
@@ -803,7 +805,7 @@ safely_chdir_nofollow (const char *dest,
 		       enum TraversalDirection direction,
 		       struct stat *statbuf_dest,
 		       enum ChdirSymlinkHandling symlink_follow_option,
-		       boolean *did_stat)
+		       bool *did_stat)
 {
   int extraflags, fd;
 
@@ -882,7 +884,7 @@ safely_chdir (const char *dest,
 	      enum TraversalDirection direction,
 	      struct stat *statbuf_dest,
 	      enum ChdirSymlinkHandling symlink_follow_option,
-	      boolean *did_stat)
+	      bool *did_stat)
 {
   enum SafeChdirStatus result;
 
@@ -921,13 +923,13 @@ safely_chdir (const char *dest,
 }
 
 
-
+
 /* Safely go back to the starting directory. */
 static void
 chdir_back (void)
 {
   struct stat stat_buf;
-  boolean dummy;
+  bool dummy;
 
   if (starting_desc < 0)
     {
@@ -943,7 +945,7 @@ chdir_back (void)
 #endif
 
       if (chdir (starting_dir) != 0)
-	fatal_file_error (starting_dir);
+	fatal_nontarget_file_error (errno, starting_dir);
 
       wd_sanity_check (starting_dir,
 		       program_name,
@@ -962,11 +964,11 @@ chdir_back (void)
 
       if (fchdir (starting_desc) != 0)
 	{
-	  fatal_file_error (starting_dir);
+	  fatal_nontarget_file_error (errno, starting_dir);
 	}
     }
 }
-
+
 /* Move to the parent of a given directory and then call a function,
  * restoring the cwd.  Don't bother changing directory if the
  * specified directory is a child of "." or is the root directory.
@@ -998,7 +1000,7 @@ at_top (char *pathname,
       enum TraversalDirection direction;
       enum SafeChdirStatus chdir_status;
       struct stat st;
-      boolean did_stat = false;
+      bool did_stat = false;
 
       dirchange = 1;
       if (0 == strcmp (base, ".."))
@@ -1070,7 +1072,7 @@ do_process_predicate (char *pathname,
 
 
 
-
+
 /* Descend PATHNAME, which is a command-line argument.
 
    Actions like -execdir assume that we are in the
@@ -1100,7 +1102,7 @@ static int dir_curr = -1;
 #define DIR_ALLOC_STEP 32
 
 
-
+
 /* We've detected a file system loop.   This is caused by one of
  * two things:
  *
@@ -1168,7 +1170,7 @@ issue_loop_warning (const char *name, const char *pathname, int level)
    Return nonzero iff PATHNAME is a directory. */
 
 static int
-process_path (char *pathname, char *name, boolean leaf, char *parent,
+process_path (char *pathname, char *name, bool leaf, char *parent,
 	      mode_t mode)
 {
   struct stat stat_buf;
@@ -1183,6 +1185,7 @@ process_path (char *pathname, char *name, boolean leaf, char *parent,
   state.type = 0;
   state.have_stat = false;
   state.have_type = false;
+  state.already_issued_stat_error_msg = false;
 
   if (!digest_mode (&mode, pathname, name, &stat_buf, leaf))
     return 0;
@@ -1271,7 +1274,7 @@ process_path (char *pathname, char *name, boolean leaf, char *parent,
   return 1;
 }
 
-
+
 /* Scan directory PATHNAME and recurse through process_path for each entry.
 
    PATHLEN is the length of PATHNAME.
@@ -1287,7 +1290,7 @@ static void
 process_dir (char *pathname, char *name, int pathlen, const struct stat *statp, char *parent)
 {
   int subdirs_left;		/* Number of unexamined subdirs in PATHNAME. */
-  boolean subdirs_unreliable;	/* if true, cannot use dir link count as subdir limif (if false, it may STILL be unreliable) */
+  bool subdirs_unreliable;	/* if true, cannot use dir link count as subdir limif (if false, it may STILL be unreliable) */
   unsigned int idx;		/* Which entry are we on? */
   struct stat stat_buf;
   size_t dircount = 0u;
@@ -1328,7 +1331,7 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
       unsigned cur_path_size;	/* Bytes allocated for `cur_path'. */
       register unsigned file_len; /* Length of each path to process. */
       register unsigned pathname_len; /* PATHLEN plus trailing '/'. */
-      boolean did_stat = false;
+      bool did_stat = false;
 
       if (pathname[pathlen - 1] == '/')
 	pathname_len = pathlen + 1; /* For '\0'; already have '/'. */
@@ -1489,7 +1492,7 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
 	  /* We could go back and do the next command-line arg
 	     instead, maybe using longjmp.  */
 	  char const *dir;
-	  boolean deref = following_links () ? true : false;
+	  bool deref = following_links () ? true : false;
 
 	  if ( (state.curdepth>0) && !deref)
 	    dir = "..";
