@@ -172,19 +172,6 @@ inside_dir (int dir_fd)
 #ifdef STAT_MOUNTPOINTS
 static void init_mounted_dev_list (void);
 #endif
-
-/* We have encountered an error which should affect the exit status.
- * This is normally used to change the exit status from 0 to 1.
- * However, if the exit status is already 2 for example, we don't want to
- * reduce it to 1.
- */
-static void
-error_severity (int level)
-{
-  if (state.exit_status < level)
-    state.exit_status = level;
-}
-
 
 #define STRINGIFY(X) #X
 #define HANDLECASE(N) case N: return #N;
@@ -214,7 +201,7 @@ get_fts_info_name (int info)
       return buf;
     }
 }
-
+
 static void
 visit (FTS *p, FTSENT *ent, struct stat *pstat)
 {
@@ -234,7 +221,7 @@ visit (FTS *p, FTSENT *ent, struct stat *pstat)
       fts_set (p, ent, FTS_SKIP);
     }
 }
-
+
 static const char*
 partial_quotearg_n (int n, char *s, size_t len, enum quoting_style style)
 {
@@ -255,7 +242,7 @@ partial_quotearg_n (int n, char *s, size_t len, enum quoting_style style)
     }
 }
 
-
+
 /* We've detected a file system loop.   This is caused by one of
  * two things:
  *
@@ -296,25 +283,21 @@ issue_loop_warning (FTSENT * ent)
 				 options.err_quoting_style));
     }
 }
-
+
 /*
  * Return true if NAME corresponds to a file which forms part of a
  * symbolic link loop.  The command
  *      rm -f a b; ln -s a b; ln -s b a
  * produces such a loop.
  */
-static boolean
+static bool
 symlink_loop (const char *name)
 {
   struct stat stbuf;
-  int rv;
-  if (following_links ())
-    rv = stat (name, &stbuf);
-  else
-    rv = lstat (name, &stbuf);
+  const int rv = options.xstat (name, &stbuf);
   return (0 != rv) && (ELOOP == errno);
 }
-
+
 
 static int
 complete_execdirs_cb (void *context)
@@ -372,9 +355,6 @@ show_outstanding_execdirs (FILE *fp)
     }
 }
 
-
-
-
 static void
 consider_visiting (FTS *p, FTSENT *ent)
 {
@@ -410,15 +390,13 @@ consider_visiting (FTS *p, FTSENT *ent)
   if (ent->fts_info == FTS_ERR
       || ent->fts_info == FTS_DNR)
     {
-      error (0, ent->fts_errno, "%s",
-	     safely_quote_err_filename (0, ent->fts_path));
-      error_severity (1);
+      nonfatal_target_file_error (ent->fts_errno, ent->fts_path);
       return;
     }
   else if (ent->fts_info == FTS_DC)
     {
       issue_loop_warning (ent);
-      error_severity (1);
+      error_severity (EXIT_FAILURE);
       return;
     }
   else if (ent->fts_info == FTS_SLNONE)
@@ -432,8 +410,7 @@ consider_visiting (FTS *p, FTSENT *ent)
        */
       if (symlink_loop (ent->fts_accpath))
 	{
-	  error (0, ELOOP, "%s", safely_quote_err_filename (0, ent->fts_path));
-	  error_severity (1);
+	  nonfatal_target_file_error (ELOOP, ent->fts_path);
 	  return;
 	}
     }
@@ -442,9 +419,7 @@ consider_visiting (FTS *p, FTSENT *ent)
       if (ent->fts_level == 0)
 	{
 	  /* e.g., nonexistent starting point */
-	  error (0, ent->fts_errno, "%s",
-		 safely_quote_err_filename (0, ent->fts_path));
-	  error_severity (1);	/* remember problem */
+	  nonfatal_target_file_error (ent->fts_errno, ent->fts_path);
 	  return;
 	}
       else
@@ -454,11 +429,24 @@ consider_visiting (FTS *p, FTSENT *ent)
 	   */
 	  if (symlink_loop (ent->fts_accpath))
 	    {
-	      error (0, ELOOP, "%s",
-		     safely_quote_err_filename (0, ent->fts_path));
-	      error_severity (1);
+	      nonfatal_target_file_error (ELOOP, ent->fts_path);
 	      return;
 	    }
+	  else
+	    {
+	      nonfatal_target_file_error (ent->fts_errno, ent->fts_path);
+	      /* Continue despite the error, as file name without stat info
+	       * might be better than not even processing the file name. This
+	       * can lead to repeated error messages later on, though, if a
+	       * predicate requires stat information.
+	       *
+	       * Not printing an error message here would be even more wrong,
+	       * though, as this could cause the contents of a directory to be
+	       * silently ignored, as the directory wouldn't be identified as
+	       * such.
+	       */
+	    }
+
 	}
     }
 
@@ -467,7 +455,7 @@ consider_visiting (FTS *p, FTSENT *ent)
       || ent->fts_info == FTS_NS /* e.g. symlink loop */)
     {
       assert (!state.have_stat);
-      assert (ent->fts_info == FTS_NSOK || state.type != 0);
+      assert (ent->fts_info == FTS_NSOK || state.type == 0);
       mode = state.type;
     }
   else
@@ -578,7 +566,7 @@ consider_visiting (FTS *p, FTSENT *ent)
 }
 
 
-
+
 static bool
 find (char *arg)
 {
@@ -615,11 +603,13 @@ find (char *arg)
     {
       error (0, errno, _("cannot search %s"),
 	     safely_quote_err_filename (0, arg));
+      error_severity (EXIT_FAILURE);
     }
   else
     {
       while ( (ent=fts_read (p)) != NULL )
 	{
+	  state.already_issued_stat_error_msg = false;
 	  state.have_stat = false;
 	  state.have_type = !!ent->fts_statp->st_mode;
 	  state.type = state.have_type ? ent->fts_statp->st_mode : 0;
@@ -634,6 +624,7 @@ find (char *arg)
 	  error (0, errno,
 		 _("failed to restore working directory after searching %s"),
 		 arg);
+	  error_severity (EXIT_FAILURE);
 	  return false;
 	}
       p = NULL;
@@ -671,7 +662,7 @@ process_all_startpoints (int argc, char *argv[])
 
 
 
-
+
 int
 main (int argc, char **argv)
 {
@@ -683,6 +674,7 @@ main (int argc, char **argv)
   else
     set_program_name ("find");
 
+  state.already_issued_stat_error_msg = false;
   state.exit_status = 0;
   state.execdirs_outstanding = false;
   state.cwd_dir_fd = AT_FDCWD;
@@ -784,7 +776,7 @@ main (int argc, char **argv)
   return state.exit_status;
 }
 
-boolean
+bool
 is_fts_enabled (int *fts_options)
 {
   /* this version of find (i.e. this main()) uses fts. */
