@@ -47,6 +47,7 @@
 #include "error.h"
 #include "verify.h"
 #include "fdleak.h"
+#include "areadlink.h"
 
 #include <selinux/selinux.h>
 
@@ -162,7 +163,7 @@
 #undef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 
-static boolean match_lname PARAMS((const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr, boolean ignore_case));
+static bool match_lname PARAMS((const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr, bool ignore_case));
 
 static char *format_date PARAMS((struct timespec ts, int kind));
 static char *ctime_format PARAMS((struct timespec ts));
@@ -237,7 +238,7 @@ struct pred_assoc pred_table[] =
   {0, "none    "}
 };
 #endif
-
+
 /* Returns ts1 - ts2 */
 static double ts_difference (struct timespec ts1,
 			     struct timespec ts2)
@@ -247,7 +248,7 @@ static double ts_difference (struct timespec ts1,
   return d;
 }
 
-
+
 static int
 compare_ts (struct timespec ts1,
 	    struct timespec ts2)
@@ -263,7 +264,7 @@ compare_ts (struct timespec ts1,
       return diff < 0.0 ? -1 : +1;
     }
 }
-
+
 /* Predicate processing routines.
 
    PATHNAME is the full pathname of the file being checked.
@@ -280,7 +281,7 @@ compare_ts (struct timespec ts1,
  * COMP_LT: before the specified time
  * COMP_EQ: after the specified time but by not more than WINDOW seconds.
  */
-static boolean
+static bool
 pred_timewindow (struct timespec ts, struct predicate const *pred_ptr, int window)
 {
   switch (pred_ptr->args.reftime.kind)
@@ -315,14 +316,14 @@ pred_timewindow (struct timespec ts, struct predicate const *pred_ptr, int windo
 }
 
 
-boolean
+bool
 pred_amin (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
   return pred_timewindow (get_stat_atime(stat_buf), pred_ptr, 60);
 }
 
-boolean
+bool
 pred_and (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   if (pred_ptr->pred_left == NULL
@@ -334,7 +335,7 @@ pred_and (const char *pathname, struct stat *stat_buf, struct predicate *pred_pt
     return false;
 }
 
-boolean
+bool
 pred_anewer (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
@@ -342,14 +343,14 @@ pred_anewer (const char *pathname, struct stat *stat_buf, struct predicate *pred
   return compare_ts (get_stat_atime(stat_buf), pred_ptr->args.reftime.ts) > 0;
 }
 
-boolean
+bool
 pred_atime (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
   return pred_timewindow (get_stat_atime(stat_buf), pred_ptr, DAYSECS);
 }
 
-boolean
+bool
 pred_closeparen (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
@@ -359,14 +360,14 @@ pred_closeparen (const char *pathname, struct stat *stat_buf, struct predicate *
   return true;
 }
 
-boolean
+bool
 pred_cmin (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
   return pred_timewindow (get_stat_ctime(stat_buf), pred_ptr, 60);
 }
 
-boolean
+bool
 pred_cnewer (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -375,7 +376,7 @@ pred_cnewer (const char *pathname, struct stat *stat_buf, struct predicate *pred
   return compare_ts (get_stat_ctime(stat_buf), pred_ptr->args.reftime.ts) > 0;
 }
 
-boolean
+bool
 pred_comma (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   if (pred_ptr->pred_left != NULL)
@@ -385,21 +386,21 @@ pred_comma (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return apply_predicate (pathname, stat_buf, pred_ptr->pred_right);
 }
 
-boolean
+bool
 pred_ctime (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
   return pred_timewindow (get_stat_ctime(stat_buf), pred_ptr, DAYSECS);
 }
 
-static boolean
+static bool
 perform_delete (int flags)
 {
   return 0 == unlinkat (state.cwd_dir_fd, state.rel_pathname, flags);
 }
 
 
-boolean
+bool
 pred_delete (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pred_ptr;
@@ -448,7 +449,7 @@ pred_delete (const char *pathname, struct stat *stat_buf, struct predicate *pred
     }
 }
 
-boolean
+bool
 pred_empty (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -459,7 +460,7 @@ pred_empty (const char *pathname, struct stat *stat_buf, struct predicate *pred_
       int fd;
       DIR *d;
       struct dirent *dp;
-      boolean empty = true;
+      bool empty = true;
 
       errno = 0;
       if ((fd = openat (state.cwd_dir_fd, state.rel_pathname, O_RDONLY
@@ -503,7 +504,7 @@ pred_empty (const char *pathname, struct stat *stat_buf, struct predicate *pred_
     return (false);
 }
 
-static boolean
+static bool
 new_impl_pred_exec (int dir_fd, const char *pathname,
 		    struct stat *stat_buf,
 		    struct predicate *pred_ptr,
@@ -566,14 +567,14 @@ new_impl_pred_exec (int dir_fd, const char *pathname,
 }
 
 
-boolean
+bool
 pred_exec (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   return new_impl_pred_exec (get_start_dirfd(),
 			     pathname, stat_buf, pred_ptr, NULL, 0);
 }
 
-boolean
+bool
 pred_execdir (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
    const char *prefix = (state.rel_pathname[0] == '/') ? NULL : "./";
@@ -583,7 +584,7 @@ pred_execdir (const char *pathname, struct stat *stat_buf, struct predicate *pre
 			      prefix, (prefix ? 2 : 0));
 }
 
-boolean
+bool
 pred_false (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
@@ -594,7 +595,7 @@ pred_false (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return (false);
 }
 
-boolean
+bool
 pred_fls (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   FILE * stream = pred_ptr->args.printf_vec.stream;
@@ -605,7 +606,7 @@ pred_fls (const char *pathname, struct stat *stat_buf, struct predicate *pred_pt
   return true;
 }
 
-boolean
+bool
 pred_fprint (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
@@ -619,7 +620,7 @@ pred_fprint (const char *pathname, struct stat *stat_buf, struct predicate *pred
   return true;
 }
 
-boolean
+bool
 pred_fprint0 (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   FILE * fp = pred_ptr->args.printf_vec.stream;
@@ -696,7 +697,7 @@ checked_fprintf (struct format_val *dest, const char *fmt, ...)
   va_start (ap, fmt);
   rv = vfprintf (dest->stream, fmt, ap);
   if (rv < 0)
-    nonfatal_file_error (dest->filename);
+    nonfatal_nontarget_file_error (errno, dest->filename);
 }
 
 
@@ -707,7 +708,7 @@ checked_print_quoted (struct format_val *dest,
   int rv = print_quoted (dest->stream, dest->quote_opts, dest->dest_is_tty,
 			 format, s);
   if (rv < 0)
-    nonfatal_file_error (dest->filename);
+    nonfatal_nontarget_file_error (errno, dest->filename);
 }
 
 
@@ -716,7 +717,7 @@ checked_fwrite (void *p, size_t siz, size_t nmemb, struct format_val *dest)
 {
   int items_written = fwrite (p, siz, nmemb, dest->stream);
   if (items_written < nmemb)
-    nonfatal_file_error (dest->filename);
+    nonfatal_nontarget_file_error (errno, dest->filename);
 }
 
 static void
@@ -724,7 +725,7 @@ checked_fflush (struct format_val *dest)
 {
   if (0 != fflush (dest->stream))
     {
-      nonfatal_file_error (dest->filename);
+      nonfatal_nontarget_file_error (errno, dest->filename);
     }
 }
 
@@ -874,14 +875,16 @@ do_fprintf (struct format_val *dest,
 
 	    if (S_ISLNK (stat_buf->st_mode))
 	      {
-		linkname = get_link_name_at (pathname, state.cwd_dir_fd, state.rel_pathname);
-		if (linkname == 0)
-		  state.exit_status = 1;
+		linkname = areadlinkat (state.cwd_dir_fd, state.rel_pathname);
+		if (linkname == NULL)
+		  {
+		    nonfatal_target_file_error (errno, pathname);
+		    state.exit_status = 1;
+		  }
 	      }
 	    if (linkname)
 	      {
 		checked_print_quoted (dest, segment->text, linkname);
-		free (linkname);
 	      }
 	    else
 	      {
@@ -890,6 +893,7 @@ do_fprintf (struct format_val *dest,
 		 */
 		checked_print_quoted (dest, segment->text, "");
 	      }
+	    free (linkname);
 	  }
 #endif				/* S_ISLNK */
 	  break;
@@ -912,7 +916,7 @@ do_fprintf (struct format_val *dest,
 	       scheme.  But help the compiler in the common case where
 	       the host uses the traditional numbering scheme.  */
 	    mode_t m = stat_buf->st_mode;
-	    boolean traditional_numbering_scheme =
+	    bool traditional_numbering_scheme =
 	      (S_ISUID == 04000 && S_ISGID == 02000 && S_ISVTX == 01000
 	       && S_IRUSR == 00400 && S_IWUSR == 00200 && S_IXUSR == 00100
 	       && S_IRGRP == 00040 && S_IWGRP == 00020 && S_IXGRP == 00010
@@ -1026,7 +1030,7 @@ do_fprintf (struct format_val *dest,
 		/* If we would normally follow links, do not do so.
 		 * If we would normally not follow links, do so.
 		 */
-		if ((following_links () ? lstat : stat)
+		if ((following_links () ? optionp_stat : optionl_stat)
 		    (state.rel_pathname, &sbuf) != 0)
 		  {
 		    if ( errno == ENOENT )
@@ -1098,7 +1102,7 @@ do_fprintf (struct format_val *dest,
     }
 }
 
-boolean
+bool
 pred_fprintf (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   struct format_val *dest = &pred_ptr->args.printf_vec;
@@ -1166,7 +1170,7 @@ pred_fprintf (const char *pathname, struct stat *stat_buf, struct predicate *pre
   return true;
 }
 
-boolean
+bool
 pred_fstype (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1177,7 +1181,7 @@ pred_fstype (const char *pathname, struct stat *stat_buf, struct predicate *pred
     return false;
 }
 
-boolean
+bool
 pred_gid (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1200,7 +1204,7 @@ pred_gid (const char *pathname, struct stat *stat_buf, struct predicate *pred_pt
   return (false);
 }
 
-boolean
+bool
 pred_group (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1211,7 +1215,7 @@ pred_group (const char *pathname, struct stat *stat_buf, struct predicate *pred_
     return (false);
 }
 
-boolean
+bool
 pred_ilname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   return match_lname (pathname, stat_buf, pred_ptr, true);
@@ -1221,10 +1225,10 @@ pred_ilname (const char *pathname, struct stat *stat_buf, struct predicate *pred
    is name to compare basename against, and FLAGS are passed to
    fnmatch.  Recall that 'find / -name /' is one of the few times where a '/'
    in the -name must actually find something. */
-static boolean
+static bool
 pred_name_common (const char *pathname, const char *str, int flags)
 {
-  boolean b;
+  bool b;
   /* We used to use last_component() here, but that would not allow us to modify the
    * input string, which is const.   We could optimise by duplicating the string only
    * if we need to modify it, and I'll do that if there is a measurable
@@ -1242,14 +1246,14 @@ pred_name_common (const char *pathname, const char *str, int flags)
   return b;
 }
 
-boolean
+bool
 pred_iname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) stat_buf;
   return pred_name_common (pathname, pred_ptr->args.str, FNM_CASEFOLD);
 }
 
-boolean
+bool
 pred_inum (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1274,7 +1278,7 @@ pred_inum (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   return (false);
 }
 
-boolean
+bool
 pred_ipath (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) stat_buf;
@@ -1284,7 +1288,7 @@ pred_ipath (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return (false);
 }
 
-boolean
+bool
 pred_links (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1307,66 +1311,71 @@ pred_links (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return (false);
 }
 
-boolean
+bool
 pred_lname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   return match_lname (pathname, stat_buf, pred_ptr, false);
 }
 
-static boolean
-match_lname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr, boolean ignore_case)
+static bool
+match_lname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr, bool ignore_case)
 {
-  boolean ret = false;
+  bool ret = false;
 #ifdef S_ISLNK
   if (S_ISLNK (stat_buf->st_mode))
     {
-      char *linkname = get_link_name_at (pathname, state.cwd_dir_fd, state.rel_pathname);
+      char *linkname = areadlinkat (state.cwd_dir_fd, state.rel_pathname);
       if (linkname)
 	{
 	  if (fnmatch (pred_ptr->args.str, linkname,
 		       ignore_case ? FNM_CASEFOLD : 0) == 0)
 	    ret = true;
-	  free (linkname);
 	}
+      else
+	{
+	  nonfatal_target_file_error (errno, pathname);
+	  state.exit_status = 1;
+	}
+      free (linkname);
     }
 #endif /* S_ISLNK */
   return ret;
 }
 
-boolean
+bool
 pred_ls (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   return pred_fls (pathname, stat_buf, pred_ptr);
 }
 
-boolean
+bool
 pred_mmin (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
   return pred_timewindow (get_stat_mtime(stat_buf), pred_ptr, 60);
 }
 
-boolean
+bool
 pred_mtime (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
   return pred_timewindow (get_stat_mtime(stat_buf), pred_ptr, DAYSECS);
 }
 
-boolean
+bool
 pred_name (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) stat_buf;
   return pred_name_common (pathname, pred_ptr->args.str, 0);
 }
 
-boolean
+bool
 pred_negate (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   return !apply_predicate (pathname, stat_buf, pred_ptr->pred_right);
 }
 
-boolean
+bool
 pred_newer (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1375,11 +1384,11 @@ pred_newer (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return compare_ts (get_stat_mtime(stat_buf), pred_ptr->args.reftime.ts) > 0;
 }
 
-boolean
+bool
 pred_newerXY (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   struct timespec ts;
-  boolean collected = false;
+  bool collected = false;
 
   assert (COMP_GT == pred_ptr->args.reftime.kind);
 
@@ -1421,7 +1430,7 @@ pred_newerXY (const char *pathname, struct stat *stat_buf, struct predicate *pre
   return compare_ts (ts, pred_ptr->args.reftime.ts) > 0;
 }
 
-boolean
+bool
 pred_nogroup (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1436,7 +1445,7 @@ pred_nogroup (const char *pathname, struct stat *stat_buf, struct predicate *pre
 #endif
 }
 
-boolean
+bool
 pred_nouser (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
 #ifdef CACHE_IDS
@@ -1454,7 +1463,7 @@ pred_nouser (const char *pathname, struct stat *stat_buf, struct predicate *pred
 }
 
 
-static boolean
+static bool
 is_ok (const char *program, const char *arg)
 {
   fflush (stdout);
@@ -1469,7 +1478,7 @@ is_ok (const char *program, const char *arg)
   return yesno ();
 }
 
-boolean
+bool
 pred_ok (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   if (is_ok (pred_ptr->args.exec_vec.replace_vec[0], pathname))
@@ -1479,7 +1488,7 @@ pred_ok (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr
     return false;
 }
 
-boolean
+bool
 pred_okdir (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   const char *prefix = (state.rel_pathname[0] == '/') ? NULL : "./";
@@ -1491,7 +1500,7 @@ pred_okdir (const char *pathname, struct stat *stat_buf, struct predicate *pred_
     return false;
 }
 
-boolean
+bool
 pred_openparen (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1500,7 +1509,7 @@ pred_openparen (const char *pathname, struct stat *stat_buf, struct predicate *p
   return true;
 }
 
-boolean
+bool
 pred_or (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   if (pred_ptr->pred_left == NULL
@@ -1512,7 +1521,7 @@ pred_or (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr
     return true;
 }
 
-boolean
+bool
 pred_path (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) stat_buf;
@@ -1521,7 +1530,7 @@ pred_path (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   return (false);
 }
 
-boolean
+bool
 pred_perm (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   mode_t mode = stat_buf->st_mode;
@@ -1590,7 +1599,7 @@ can_access (int access_type)
 }
 
 
-boolean
+bool
 pred_executable (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1600,7 +1609,7 @@ pred_executable (const char *pathname, struct stat *stat_buf, struct predicate *
   return can_access (X_OK);
 }
 
-boolean
+bool
 pred_readable (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1610,7 +1619,7 @@ pred_readable (const char *pathname, struct stat *stat_buf, struct predicate *pr
   return can_access (R_OK);
 }
 
-boolean
+bool
 pred_writable (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1620,7 +1629,7 @@ pred_writable (const char *pathname, struct stat *stat_buf, struct predicate *pr
   return can_access (W_OK);
 }
 
-boolean
+bool
 pred_print (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) stat_buf;
@@ -1633,13 +1642,13 @@ pred_print (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return true;
 }
 
-boolean
+bool
 pred_print0 (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   return pred_fprint0(pathname, stat_buf, pred_ptr);
 }
 
-boolean
+bool
 pred_prune (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1659,7 +1668,7 @@ pred_prune (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return true;
 }
 
-boolean
+bool
 pred_quit (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1677,7 +1686,7 @@ pred_quit (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   exit (state.exit_status);	/* 0 for success, etc. */
 }
 
-boolean
+bool
 pred_regex (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   int len = strlen (pathname);
@@ -1688,7 +1697,7 @@ pred_regex (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   return (false);
 }
 
-boolean
+bool
 pred_size (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   uintmax_t f_val;
@@ -1714,7 +1723,7 @@ pred_size (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   return (false);
 }
 
-boolean
+bool
 pred_samefile (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   /* Potential optimisation: because of the loop protection, we always
@@ -1755,7 +1764,7 @@ pred_samefile (const char *pathname, struct stat *stat_buf, struct predicate *pr
     }
 }
 
-boolean
+bool
 pred_true (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1764,7 +1773,7 @@ pred_true (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   return true;
 }
 
-boolean
+bool
 pred_type (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   mode_t mode;
@@ -1815,7 +1824,7 @@ pred_type (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
     return (false);
 }
 
-boolean
+bool
 pred_uid (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1837,7 +1846,7 @@ pred_uid (const char *pathname, struct stat *stat_buf, struct predicate *pred_pt
   return (false);
 }
 
-boolean
+bool
 pred_used (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   struct timespec delta, at, ct;
@@ -1857,7 +1866,7 @@ pred_used (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   return pred_timewindow (delta, pred_ptr, DAYSECS);
 }
 
-boolean
+bool
 pred_user (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) pathname;
@@ -1867,7 +1876,7 @@ pred_user (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
     return (false);
 }
 
-boolean
+bool
 pred_xtype (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   struct stat sbuf;		/* local copy, not stat_buf because we're using a different stat method */
@@ -1906,7 +1915,7 @@ pred_xtype (const char *pathname, struct stat *stat_buf, struct predicate *pred_
 }
 
 
-boolean
+bool
 pred_context (const char *pathname, struct stat *stat_buf,
 	      struct predicate *pred_ptr)
 {
@@ -1925,7 +1934,7 @@ pred_context (const char *pathname, struct stat *stat_buf,
   return rv;
 }
 
-
+
 /*  1) fork to get a child; parent remembers the child pid
     2) child execs the command requested
     3) parent waits for child; checks for proper pid of child
@@ -1945,10 +1954,10 @@ pred_context (const char *pathname, struct stat *stat_buf,
     Otherwise return false, possibly printing an error message. */
 
 
-static boolean
-prep_child_for_exec (boolean close_stdin, int dir_fd)
+static bool
+prep_child_for_exec (bool close_stdin, int dir_fd)
 {
-  boolean ok = true;
+  bool ok = true;
   if (close_stdin)
     {
       const char inputfile[] = "/dev/null";
@@ -2107,7 +2116,7 @@ launch (struct buildcmd_control *ctl, void *usercontext, int argc, char **argv)
 }
 
 
-static boolean
+static bool
 scan_for_digit_differences (const char *p, const char *q,
 			    size_t *first, size_t *n)
 {
@@ -2448,7 +2457,7 @@ ctime_format (struct timespec ts)
       return format_date (ts, '@');
     }
 }
-
+
 /* Copy STR into BUF and trim blanks from the end of BUF.
    Return BUF. */
 
@@ -2468,7 +2477,7 @@ blank_rtrim (str, buf)
   buf[++i] = '\0';
   return (buf);
 }
-
+
 /* Print out the predicate list starting at NODE. */
 void
 print_list (FILE *fp, struct predicate *node)
@@ -2484,7 +2493,7 @@ print_list (FILE *fp, struct predicate *node)
     }
   fprintf (fp, "\n");
 }
-
+
 /* Print out the predicate list starting at NODE. */
 static void
 print_parenthesised (FILE *fp, struct predicate *node)
@@ -2514,7 +2523,7 @@ print_parenthesised (FILE *fp, struct predicate *node)
 	}
     }
 }
-
+
 void
 print_optlist (FILE *fp, const struct predicate *p)
 {
@@ -2544,7 +2553,7 @@ print_optlist (FILE *fp, const struct predicate *p)
       print_parenthesised (fp, p->pred_right);
     }
 }
-
+
 void show_success_rates (const struct predicate *p)
 {
   if (options.debug_options & DebugSuccessRates)
@@ -2557,7 +2566,7 @@ void show_success_rates (const struct predicate *p)
 
 
 
-
+
 #ifdef _NDEBUG
 /* If _NDEBUG is defined, the assertions will do nothing.   Hence
  * there is no point in having a function body for pred_sanity_check()
