@@ -52,7 +52,7 @@
 # define N_(String) String
 #endif
 
-
+
 struct debug_option_assoc
 {
   char *name;
@@ -73,7 +73,7 @@ static struct debug_option_assoc debugassoc[] =
 
 
 
-
+
 /* Add a primary of predicate type PRED_FUNC (described by ENTRY) to the predicate input list.
 
    Return a pointer to the predicate node just inserted.
@@ -135,7 +135,7 @@ insert_primary_noarg (const struct parser_table *entry)
 }
 
 
-
+
 static void
 show_valid_debug_options (FILE *fp, int full)
 {
@@ -158,7 +158,7 @@ show_valid_debug_options (FILE *fp, int full)
 	}
     }
 }
-
+
 void
 usage (FILE *fp, int status, char *msg)
 {
@@ -171,7 +171,7 @@ usage (FILE *fp, int status, char *msg)
   if (0 != status)
     exit (status);
 }
-
+
 void
 set_stat_placeholders (struct stat *p)
 {
@@ -189,7 +189,7 @@ set_stat_placeholders (struct stat *p)
 #endif
 }
 
-
+
 /* Get the stat information for a file, if it is
  * not already known.  Returns 0 on success.
  */
@@ -210,15 +210,14 @@ get_statinfo (const char *pathname, const char *name, struct stat *p)
 	      /* Savannah bug #16378. */
 	      error (0, 0, _("WARNING: file %s appears to have mode 0000"),
 		     quotearg_n_style (0, options.err_quoting_style, name));
+	      error_severity (1);
 	    }
 	}
       else
 	{
 	  if (!options.ignore_readdir_race || (errno != ENOENT) )
 	    {
-	      error (0, errno, "%s",
-		     safely_quote_err_filename (0, pathname));
-	      state.exit_status = 1;
+	      nonfatal_target_file_error (errno, pathname);
 	    }
 	  return -1;
 	}
@@ -229,7 +228,7 @@ get_statinfo (const char *pathname, const char *name, struct stat *p)
 
   return 0;
 }
-
+
 /* Get the stat/type/inode information for a file, if it is not
  * already known.   Returns 0 on success (or if we did nothing).
  */
@@ -238,7 +237,7 @@ get_info (const char *pathname,
 	  struct stat *p,
 	  struct predicate *pred_ptr)
 {
-  boolean todo = false;
+  bool todo = false;
 
   /* If we need the full stat info, or we need the type info but don't
    * already have it, stat the file now.
@@ -271,6 +270,10 @@ get_info (const char *pathname,
     {
       int result = get_statinfo (pathname, state.rel_pathname, p);
       if (result != 0)
+	{
+	  return -1;		/* failure. */
+	}
+      else
 	{
 	  /* Verify some postconditions.  We can't check st_mode for
 	     non-zero-ness because of Savannah bug #16378 (which is
@@ -283,10 +286,6 @@ get_info (const char *pathname,
 	    {
 	      assert (p->st_ino);
 	    }
-	  return -1;		/* failure. */
-	}
-      else
-	{
 	  return 0;		/* success. */
 	}
     }
@@ -295,11 +294,11 @@ get_info (const char *pathname,
       return 0;			/* success; nothing to do. */
     }
 }
-
+
 /* Determine if we can use O_NOFOLLOW.
  */
 #if defined O_NOFOLLOW
-boolean
+bool
 check_nofollow (void)
 {
   struct utsname uts;
@@ -335,7 +334,7 @@ check_nofollow (void)
 #endif
 
 
-
+
 /* Examine the predicate list for instances of -execdir or -okdir
  * which have been terminated with '+' (build argument list) rather
  * than ';' (singles only).  If there are any, run them (this will
@@ -383,7 +382,7 @@ complete_pending_execdirs (int dir_fd)
 }
 
 
-
+
 /* Examine the predicate list for instances of -exec which have been
  * terminated with '+' (build argument list) rather than ';' (singles
  * only).  If there are any, run them (this will have no effect if
@@ -418,7 +417,7 @@ complete_pending_execs (struct predicate *p)
 
   complete_pending_execs (p->pred_right);
 }
-
+
 static void
 traverse_tree (struct predicate *tree,
 			  void (*callback)(struct predicate*))
@@ -451,16 +450,16 @@ undangle_file_pointers (struct predicate *p)
 
 /* Return nonzero if file descriptor leak-checking is enabled.
  */
-int
+bool
 fd_leak_check_is_enabled (void)
 {
   if (getenv ("GNU_FINDUTILS_FD_LEAK_CHECK"))
-    return 1;
+    return true;
   else
-    return 0;
+    return false;
 
 }
-
+
 /* Complete any outstanding commands.
  * Flush and close any open files.
  */
@@ -486,7 +485,7 @@ cleanup (void)
     }
 
   if (fflush (stdout) == EOF)
-    nonfatal_file_error ("standard output");
+    nonfatal_nontarget_file_error (errno, "standard output");
 }
 
 /* Savannah bug #16378 manifests as an assertion failure in pred_type()
@@ -536,7 +535,7 @@ static int hook_fstatat (int fd, const char *name, struct stat *p, int flags)
 # define fstatat (fd,name,p,flags) hook_fstatat((fd),(name),(p),(flags))
 #endif
 
-
+
 static int
 fallback_stat (const char *name, struct stat *p, int prev_rv)
 {
@@ -564,7 +563,7 @@ fallback_stat (const char *name, struct stat *p, int prev_rv)
     }
 }
 
-
+
 /* optionh_stat() implements the stat operation when the -H option is
  * in effect.
  *
@@ -600,7 +599,7 @@ optionh_stat (const char *name, struct stat *p)
       return fstatat (state.cwd_dir_fd, name, p, AT_SYMLINK_NOFOLLOW);
     }
 }
-
+
 /* optionl_stat() implements the stat operation when the -L option is
  * in effect.  That option makes us examine the thing the symbolic
  * link points to, not the symbolic link itself.
@@ -619,7 +618,7 @@ optionl_stat(const char *name, struct stat *p)
   else
     return fallback_stat (name, p, rv);
 }
-
+
 /* optionp_stat() implements the stat operation when the -P option is
  * in effect (this is also the default).  That option makes us examine
  * the symbolic link itself, not the thing it points to.
@@ -631,7 +630,7 @@ optionp_stat (const char *name, struct stat *p)
   set_stat_placeholders (p);
   return fstatat (state.cwd_dir_fd, name, p, AT_SYMLINK_NOFOLLOW);
 }
-
+
 
 static uintmax_t stat_count = 0u;
 
@@ -655,31 +654,31 @@ debug_stat (const char *file, struct stat *bufp)
   return -1;
 }
 
-
-int
+
+bool
 following_links(void)
 {
   switch (options.symlink_handling)
     {
     case SYMLINK_ALWAYS_DEREF:
-      return 1;
+      return true;
     case SYMLINK_DEREF_ARGSONLY:
       return (state.curdepth == 0);
     case SYMLINK_NEVER_DEREF:
     default:
-      return 0;
+      return false;
     }
 }
 
-
+
 /* Take a "mode" indicator and fill in the files of 'state'.
  */
-int
+bool
 digest_mode (mode_t *mode,
 	     const char *pathname,
 	     const char *name,
 	     struct stat *pstat,
-	     boolean leaf)
+	     bool leaf)
 {
   /* If we know the type of the directory entry, and it is not a
    * symbolic link, we may be able to avoid a stat() or lstat() call.
@@ -690,7 +689,7 @@ digest_mode (mode_t *mode,
 	{
 	  /* mode is wrong because we should have followed the symlink. */
 	  if (get_statinfo (pathname, name, pstat) != 0)
-	    return 0;
+	    return false;
 	  *mode = state.type = pstat->st_mode;
 	  state.have_type = true;
 	}
@@ -715,7 +714,7 @@ digest_mode (mode_t *mode,
       else
 	{
 	  if (get_statinfo (pathname, name, pstat) != 0)
-	    return 0;
+	    return false;
 
 	  /* If -L is in effect and we are dealing with a symlink,
 	   * st_mode is the mode of the pointed-to file, while mode is
@@ -728,15 +727,15 @@ digest_mode (mode_t *mode,
     }
 
   /* success. */
-  return 1;
+  return true;
 }
 
-
+
 /* Return true if there are no predicates with no_default_print in
    predicate list PRED, false if there are any.
    Returns true if default print should be performed */
 
-boolean
+bool
 default_prints (struct predicate *pred)
 {
   while (pred != NULL)
@@ -748,8 +747,8 @@ default_prints (struct predicate *pred)
   return (true);
 }
 
-boolean
-looks_like_expression (const char *arg, boolean leading)
+bool
+looks_like_expression (const char *arg, bool leading)
 {
   switch (arg[0])
     {
@@ -781,14 +780,14 @@ looks_like_expression (const char *arg, boolean leading)
       return false;
     }
 }
-
+
 static void
 process_debug_options (char *arg)
 {
   const char *p;
   char *token_context = NULL;
   const char delimiters[] = ",";
-  boolean empty = true;
+  bool empty = true;
   size_t i;
 
   p = strtok_r (arg, delimiters, &token_context);
@@ -822,7 +821,7 @@ process_debug_options (char *arg)
     }
 }
 
-
+
 static void
 process_optimisation_option (const char *arg)
 {
@@ -881,7 +880,7 @@ process_optimisation_option (const char *arg)
 	}
     }
 }
-
+
 int
 process_leading_options (int argc, char *argv[])
 {
@@ -931,7 +930,7 @@ process_leading_options (int argc, char *argv[])
     }
   return end_of_leading_options;
 }
-
+
 static struct timespec
 now(void)
 {
@@ -951,7 +950,7 @@ now(void)
   retval.tv_nsec = 0;
   return retval;
 }
-
+
 void
 set_option_defaults (struct options *p)
 {
@@ -1028,20 +1027,21 @@ set_option_defaults (struct options *p)
   p->err_quoting_style = locale_quoting_style;
 }
 
-
+
 /* get_start_dirfd
  *
  * Returns the fd for the directory we started in.
  */
-int get_start_dirfd (void)
+int
+get_start_dirfd (void)
 {
   return starting_desc;
 }
-
+
 /* apply_predicate
  *
  */
-boolean
+bool
 apply_predicate(const char *pathname, struct stat *stat_buf, struct predicate *p)
 {
   ++p->perf.visits;
@@ -1063,7 +1063,7 @@ apply_predicate(const char *pathname, struct stat *stat_buf, struct predicate *p
     }
 }
 
-
+
 /* safely_quote_err_filename
  *
  */
@@ -1072,36 +1072,84 @@ safely_quote_err_filename (int n, char const *arg)
 {
   return quotearg_n_style (n, options.err_quoting_style, arg);
 }
-
+
+/* We have encountered an error which should affect the exit status.
+ * This is normally used to change the exit status from 0 to 1.
+ * However, if the exit status is already 2 for example, we don't want to
+ * reduce it to 1.
+ */
+void
+error_severity (int level)
+{
+  if (state.exit_status < level)
+    state.exit_status = level;
+}
+
+
 /* report_file_err
  */
 static void
-report_file_err(int exitval, int errno_value, const char *name)
+report_file_err(int exitval, int errno_value,
+		bool is_target_file, const char *name)
 {
   /* It is important that the errno value is passed in as a function
    * argument before we call safely_quote_err_filename(), because otherwise
    * we might find that safely_quote_err_filename() changes errno.
    */
-  if (state.exit_status < 1)
-    state.exit_status = 1;
-
-  error (exitval, errno_value, "%s", safely_quote_err_filename (0, name));
+  if (!is_target_file || !state.already_issued_stat_error_msg)
+    {
+      error (exitval, errno_value, "%s", safely_quote_err_filename (0, name));
+      error_severity (1);
+    }
+  if (is_target_file)
+    {
+      state.already_issued_stat_error_msg = true;
+    }
 }
-
-/* fatal_file_error
+
+/* nonfatal_target_file_error
+ */
+void
+nonfatal_target_file_error (int errno_value, const char *name)
+{
+  report_file_err (0, errno_value, true, name);
+}
+
+/* fatal_target_file_error
+ *
+ * Report an error on a target file (i.e. a file we are searching).
+ * Such errors are only reported once per searched file.
  *
  */
 void
-fatal_file_error(const char *name)
+fatal_target_file_error(int errno_value, const char *name)
 {
-  report_file_err (1, errno, name);
+  report_file_err (1, errno_value, true, name);
   /*NOTREACHED*/
   abort ();
 }
 
+/* nonfatal_nontarget_file_error
+ *
+ */
 void
-nonfatal_file_error (const char *name)
+nonfatal_nontarget_file_error (int errno_value, const char *name)
 {
-  report_file_err (0, errno, name);
+  report_file_err (0, errno_value, false, name);
 }
 
+/* fatal_nontarget_file_error
+ *
+ */
+void
+fatal_nontarget_file_error(int errno_value, const char *name)
+{
+  /* We're going to exit fatally, so make sure we always isssue the error
+   * message, even if it will be duplicate.   Motivation: otherwise it may
+   * not be clear what went wrong.
+   */
+  state.already_issued_stat_error_msg = false;
+  report_file_err (1, errno_value, false, name);
+  /*NOTREACHED*/
+  abort ();
+}
