@@ -30,6 +30,7 @@
 #include <string.h>
 #include <wchar.h>
 #include <locale.h>
+#include <stdbool.h>
 
 #if ENABLE_NLS
 # include <libintl.h>
@@ -443,36 +444,6 @@ bc_push_arg (struct buildcmd_control *ctl,
     }
 }
 
-#if 0
-/* We used to set posix_arg_size_min to the LINE_MAX limit, but
- * currently we use _POSIX_ARG_MAX (which is the minimum value).
- */
-static size_t
-get_line_max (void)
-{
-  long val;
-#ifdef _SC_LINE_MAX
-  val = sysconf (_SC_LINE_MAX);
-#else
-  val = -1;
-#endif
-
-  if (val > 0)
-    return val;
-
-  /* either _SC_LINE_MAX was not available or
-   * there is no particular limit.
-   */
-#ifdef LINE_MAX
-  val = LINE_MAX;
-#endif
-
-  if (val > 0)
-    return val;
-
-  return 2048L;			/* a reasonable guess. */
-}
-#endif
 
 size_t
 bc_get_arg_max (void)
@@ -683,7 +654,7 @@ exceeds (const char *env_var_name, size_t quantity)
 }
 
 /* Return nonzero if the indicated argument list exceeds a testing limit. */
-int
+bool
 bc_args_exceed_testing_limit (const char **argv)
 {
   size_t chars, args;
