@@ -70,7 +70,7 @@
 #include <getopt.h>
 #include <xstrtol.h>
 
-#include <stdbool.h>		/* for bool/boolean */
+#include <stdbool.h>
 
 /* The presence of unistd.h is assumed by gnulib these days, so we
  * might as well assume it too.
@@ -138,8 +138,6 @@
 # define TOLOWER(Ch) (isupper (Ch) ? tolower (Ch) : (Ch))
 #endif
 
-/* typedef enum {false, true} boolean; */
-
 /* Warn if a database is older than this.  8 days allows for a weekly
    update that takes up to a day to perform.  */
 static unsigned int warn_number_units = 8;
@@ -1095,11 +1093,7 @@ search_one_database (int argc,
   lastinspector = NULL;
   past_pat_inspector = NULL;
   results_were_filtered = false;
-#if 0
-  procdata.pathsize = 1026;	/* Increased as necessary by locate_read_str.  */
-#else
   procdata.pathsize = 128;	/* Increased as necessary by locate_read_str.  */
-#endif
   procdata.original_filename = xmalloc (procdata.pathsize);
 
 
@@ -1536,7 +1530,7 @@ drop_privs (void)
       /* deliberate infinite loop */
     }
 }
-
+
 static int
 opendb (const char *name)
 {
@@ -1556,7 +1550,7 @@ opendb (const char *name)
     }
   return fd;
 }
-
+
 int
 dolocate (int argc, char **argv, int secure_db_fd)
 {
@@ -1901,7 +1895,7 @@ dolocate (int argc, char **argv, int secure_db_fd)
   else
     return 1;
 }
-
+
 #define ARRAYSIZE(a) (sizeof (a)/sizeof (a[0]))
 static int
 open_secure_db (void)
